#!/usr/bin/perl
#----------------------------------------------------------------------------------------
#	moldyext2.pl
#
#	An alternate of moldyext which is distributed with the moldy package
#
#		Masato YOSHIYA		May/02/2002		rewritten thoroughoutly
#		Masato YOSHIYA		Apr/26/2004		Modified for $NumSpecies>=4
#----------------------------------------------------------------------------------------

	# definition
	$MaxNumArgv=10;

	# Analyze argument values and store mode in $mode, file names in @file_list and
	# field numbers in @field_list.
	&anal_argv();

	# determine number of molecules (in case of solid, number of atom species)
	open(IN,$file_list[0]) || die "Cannot open $file_list[0] to read: $!\n";
	$NumSpecies=&count_species();
	# determine which column and which row should be read based on @field_list;
	&anal_field();
	close(IN);

	foreach $file (@file_list){
		open(IN,$file) || die "Cannot open $file to read: $!\n";
		&find_value();
		close(IN);
	}

	exit(0);

#----------------------------------------------------------------------------------------
sub anal_argv	{
	if($#ARGV>=$MaxNumArgv){
		die "Too many argument values!\n";
	}
	$nox=0;
	while($#ARGV>=0){
		$elements=shift(@ARGV);
		if($elements eq "-f"){
			$field_elements=shift(@ARGV);
			&anal_field_list();
		}
		elsif(($elements eq "-h")||($elements eq "--help")){
			&usage();
		}
		elsif($elements eq "-m"){
			$mode=shift(@ARGV);
		}
		elsif($elements eq "-nox"){
			$nox=1;
		}
		else{
			if($elements =~ /^-/){
				&usage();
			}
			push(@file_list,$elements);
		}
	}
}
#----------------------------------------------------------------------------------------
sub usage	{
	print STDERR "\n";
	print STDERR "Usage:  moldyext2.pl -m <mode> -f <field list> [-nox] <file1> <file2> ....\n";
	print STDERR "Descr.: Field list must be provided in the same form as in moldyext.\n";
	print STDERR "        Mode number must be provided by a number:\n";
	print STDERR "            1: Instantaneous values\n";
	print STDERR "            2: Rolling average values\n";
	print STDERR "            3: Standard deviation of the rolling average\n";
	print STDERR "            4: Averages given every 5000 steps(default) or so\n";
	print STDERR "            5: Standard deviation of No.4\n";
	print STDERR "        Time is not output when the option -nox is specified.\n";
	print STDERR "\n";
	exit(1);
}
#----------------------------------------------------------------------------------------
sub anal_field_list	{
	my $FirstNum,$LastNum,$i;
	@values=split(/,/,$field_elements);
	foreach $in (@values){
		if($in =~ /^[0-9]+$/){	# if $in is just a number, not a conbination of numbers
			push(@field_list,$in);
		}
		elsif($in =~ /-/){
			($FirstNum,$LastNum)=split(/-/,$in);
			for($i=$FirstNum;$i<=$LastNum;$i++){
				push(@field_list,$i);
			}
		}
		else{
			die "Unknown field number !\n";
		}
	}
}
#----------------------------------------------------------------------------------------
sub moldyout_read {
	chomp($line=<IN>);
	if($line =~ /^/){
		chomp($line=<IN>);
	}
	return $line;
}
#----------------------------------------------------------------------------------------
sub count_species {
	$NumSpecies=0;
	$found=0;
	while($found==0){
		$line=&moldyout_read();
		if($line=~/ System specification read in/){
			$found=1;
		}
	}
	$found=0;
	while($found==0){
		$line=&moldyout_read();
		if(($line=~/Number of molecules/)||($line =~ /Number of ions/)){
			$NumSpecies++;
			if($line=~/Number of ions/){
				$DifferentFormat=1;
			}
			else{
				$DifferentFormat=0;
			}
		}
		if($line eq ""){
			$found=1;
		}
	}
	return $NumSpecies;
}

#----------------------------------------------------------------------------------------
sub anal_field	{

	foreach $field (@field_list){

		if($DifferentFormat==0){
			if($NumSpecies==1){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=19)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=20)&&($field<=25)){
					$Row=3;
					$Column=$field-19;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies==2){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=24)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=25)&&($field<=30)){
					$Row=3;
					$Column=$field-24;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies==3){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=24)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=25)&&($field<=34)){
					$Row=3;
					$Column=$field-24;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies>=4){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=24)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=25)&&($field<=34)){
					$Row=3;
					$Column=$field-24;
				}
				elsif($field>=35){
					for($j=4;j<=$NumSpecies;$j++){
						$LowLimit=34+4*($j-3-1)+1;
						$HighLimit=34+4*($j-3);
						if(($field>=$LowLimit)&&($field<=$HighLimit)){
							$Row=$j;
							$Column=$field-(34+4*($j-3));
						}
					}
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			else{
				die "You must modify the program!\n";
			}
			$Field=$Row.",".$Column;
			push(@Field_List,$Field);
		}
		elsif($DifferentFormat==1){
			# !!! for compatibility for older version of moldy
			if($field>17){
				$field+=1;
			}
			if($NumSpecies==1){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=19)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=20)&&($field<=26)){
					$Row=3;
					$Column=$field-19;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies==2){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=25)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=26)&&($field<=30)){
					$Row=3;
					$Column=$field-25;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies==3){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=25)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=26)&&($field<=34)){
					$Row=3;
					$Column=$field-25;
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			elsif($NumSpecies>=4){
				if($field <= 13){
					$Row=1;
					$Column=$field;
				}
				elsif(($field>=14)&&($field<=25)){
					$Row=2;
					$Column=$field-13;
				}
				elsif(($field>=26)&&($field<=35)){
					$Row=3;
					$Column=$field-25;
				}
				elsif($field>=36){
					for($j=4;j<=$NumSpecies;$j++){
						$LowLimit=35+4*($j-3-1)+1;
						$HighLimit=35+4*($j-3);
						if(($field>=$LowLimit)&&($field<=$HighLimit)){
							$Row=$j;
							$Column=$field-(35+4*($j-3));
						}
					}
				}
				else{
					die "Invalid field: $field\n";
				}
			}
			else{
				die "You must modify the program!\n";
			}
			$Field=$Row.",".$Column;
			push(@Field_List,$Field);
		}
	}
}
#----------------------------------------------------------------------------------------
sub find_value {

	if($NumSpecies<=3){
		$NumLines=3;
	}
	else{
		$NumLines=$NumSpecies;
	}

	if($mode<=3){
		while(!eof(IN)){
			$found=0;
			while($found==0){
				$line=&moldyout_read();
				if($line =~ /======== Timestep/){
					chomp($line);
					@values=split(/ +/,$line);
					$TimeStep=$values[2];
					if($nox==0){
						print STDOUT $TimeStep;
					}
					$found=1;
				}
				if(eof(IN)){
					return;
				}
			}
			if($mode == 3){
				for($i=0;$i<(($NumLines+1)*2);$i++){	&moldyout_read();	}
			}
			elsif($mode == 2){
				for($i=0;$i<($NumLines+1);$i++){	&moldyout_read();	}
			}
			for($j=1;$j<=$NumLines;$j++){
				$total=1;
				chomp($line=&moldyout_read());
				@values=split(/ +/,$line);
				for($i=1;$i<=$#values;$i++){
					$CurrentField=$j.",".$total;
					foreach $field (@Field_List){
						if($field eq $CurrentField){
							print STDOUT "\t",$values[$i];
						}
					}
					$total++;
				}
			}
			print STDOUT "\n";
		}
	}
	else{
		die "subroutine for mode 4 or 5 is under construction.\n";
	}
}
#----------------------------------------------------------------------------------------
